from enum import Enum


class Backend(str, Enum):
    mock = "mock"
    vllm = "vllm"
    litellm = "litellm"


def init_backend(backend_type, model_name):
    match backend_type:
        case "mock":
            from .mock import MockLLM
            return MockLLM()
        case "vllm":
            from .vllm import VLLMBackend
            return VLLMBackend(model_path=model_name)
        case "litellm":
            from .litellm import LiteLLMBackend
            return LiteLLMBackend(model_name=model_name)
        case _:
            raise ValueError(f"Unknown backend type: {backend_type}")