from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any


@dataclass
class Instance:
    question: str
    gold_completion: Optional[str] = None
    choices: Optional[list[str]] = None
    solution: Optional[str] = None
    subset: Optional[str] = None
    metadata: dict = field(default_factory=dict)


@dataclass
class LMOutput:
    text: str
    logprobs: Optional[List[Dict[str, Any]]] = None
    extracted_answer: Optional[str] = None


@dataclass
class Response:
    input: Instance
    messages: dict
    request: str
    output: LMOutput


class Task:
    HF_PATH: str
    SUBSETS: Optional[list[str]] = None
    INSTRUCTION: Optional[str] = None
    FEW_SHOT: Optional[list] = None
    EXTRACTOR: Optional[callable] = lambda gen: gen
    SEED: Optional[int] = None

    def _process_instance(self, doc):
        raise NotImplementedError()

    def extract_outputs(self, generations: list[LMOutput]):
        for gen in generations:
            gen.extracted_answer = self.__class__.EXTRACTOR(gen.text)
        return generations
