from minieval.datatypes import Instance


def base_model_chat_template(messages, tokenize=False):
    assert not tokenize
    assert len(messages) == 1
    message = messages[0]["content"]
    prompt = f"Question: {message}\nAnswer:"
    return prompt


class Formatter:
    def __init__(self, template_func):
        self.template_func = template_func

    def build_messages(self, instances):
        return list(map(self._build_message, instances))
    
    def build_requests(self, messages):
        return list(map(self._build_request, messages))

    def _build_message(self, instance: Instance) -> dict:
        messages = self._build_message(instance)
        return messages
    
    def _build_request(self, messages: dict) -> str:
        request = self.template_func(messages, tokenize=False)
        return request

    def _build_message(self):
        raise NotImplementedError()


class CoT(Formatter):
    def _build_message(self, instance: Instance):
        messages = [{"role": "user", "content": instance.question}]
        return messages


class MC(Formatter):
    def _build_message(self, instance: Instance):
        choices_text = \
            "\n".join(f"{chr(65+i)}. {choice}" for i, choice in enumerate(instance.choices))
        question_text = f"{instance.question}\nChoices:\n{choices_text}"
        messages = [{"role": "user", "content": question_text}]
        return messages


class RC(Formatter):
    def _build_message(self, instance: Instance):
        choices_text = \
            "\n".join(f"{chr(65+i)}. {choice}" for i, choice in enumerate(instance.choices))
        question_text = f"{instance.question}\nChoices:\n{choices_text}"
        messages = [{"role": "user", "content": question_text}]
        return messages
