from datasets import load_dataset

from minieval.datatypes import Instance, Task
from minieval.extract import math_latex


class AIME(Task):
    HF_PATH = "allenai/aime-2021-2025"
    EXTRACTOR = math_latex.extract_math_answer
    INSTRUCTION = "Present the answer in LaTex format: \\boxed{Your answer}"

    TASK_CONFIG_DEFAULTS = {
        "generation_kwargs": {
            "max_gen_toks": 4096,
            "temperature": 0.0,
            "truncate_context": False,
            "do_sample": False,
        },
    }

    def __init__(self):
        dataset = load_dataset(path=self.HF_PATH, split="train")
        self.requests = map(self._process_doc, dataset)

    def _process_doc(self, doc):
        problem_from = doc.get("url").split("/")[-2]
        year = problem_from.split("_")[0]
        aime_number = "AIME_" + problem_from.split("_")[2]

        return Instance(
            question=doc["problem"],
            gold_completion=doc["solution"],
            solution=doc["answer"],
            subset=year,
            metadata={
                "id": aime_number,
                "year": year,
            },
        )
