from datasets import load_dataset

from minieval.datatypes import Instance, Task
from minieval.few_shot import FewShot


class ARC(Task):
    HF_PATH = "ai2_arc"

    def __init__(self, dataset_name):
        if type(self) is ARC:
            raise TypeError("ARC is an abstract class. Please use a child!")

        requests = []
        for subset in ["train", "validation", "test"]:
            dataset = load_dataset(path=self.HF_PATH, split=subset, name=dataset_name)
            requests += map(self._process_instance, dataset)
        self.requests = requests

    def _process_instance(self, doc):
        if doc["answerKey"].isdigit():
            doc["answerKey"] = chr(ord("A") + int(doc["answerKey"]) - 1)

        gold_idx = ["A", "B", "C", "D", "E"].index(doc["answerKey"])
        choices = doc["choices"]["text"]

        return Instance(
            question=doc["question"],
            gold_completion=choices[gold_idx],
            choices=choices,
            solution=gold_idx,
            metadata={"id": doc["id"]},
        )


class ARCChallenge(ARC):
    FEW_SHOT = FewShot.get("arc_challenge::olmes")

    def __init__(self):
        super().__init__(dataset_name="ARC-Challenge")


class ARCEasy(ARC):
    FEW_SHOT = FewShot.get("arc_easy::olmes")

    def __init__(self):
        super().__init__(dataset_name="ARC-Easy")
